<?php
/**
* @package RSMediaGallery!
* @copyright (C) 2011 - 2021 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
*/

defined('_JEXEC') or die('Restricted access');

class JFormFieldHrspacer extends JFormField
{
	protected $type = 'Hrspacer';
	
	public function __construct($parent = null) 
	{
		parent::__construct($parent);
		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/version.php';
		
        $version = (string) new RSMediaGalleryVersion;
		
		JHtml::_('stylesheet', 'com_rsmediagallery/hrspacer.css', array('relative' => true, 'version' => $version));
	}
	
	protected function getInput()
	{
		return ' ';
	}

	protected function getLabel()
	{
		$html = array();
		$class = !empty($this->class) ? ' class="' . $this->class . '"' : '';
		$html[] = '<span class="hrspacer">';
		$html[] = '<span class="before"></span>';
		$html[] = '<span' . $class . '>';
		
		
		$label = '';

		// Get the label text from the XML element, defaulting to the element name.
		$text = $this->element['label'] ? (string) $this->element['label'] : (string) $this->element['name'];
		$text = $this->translateLabel ? JText::_($text) : $text;

		// Build the class for the label.
		$class = !empty($this->description) ? 'hasPopover' : '';
		$class = $this->required == true ? $class . ' required' : $class;

		// Add the opening label tag and main attributes attributes.
		$label .= '<label id="' . $this->id . '-lbl" class="' . $class . '"';
		
		// Add the label text and closing tag.
		$label .= '>' . $text . '</label>';
		$html[] = $label;
		
		if ((string) $this->element['hr'] === 'true')
		{
			$html[] = '<hr>';
		}

		$html[] = '</span>';
		$html[] = '<span class="after"></span>';
		$html[] = '</span>';

		return implode('', $html);
	}

	/**
	 * Method to get the field title.
	 *
	 * @return  string  The field title.
	 *
	 * @since   1.7.0
	 */
	protected function getTitle()
	{
		return $this->getLabel();
	}

	/**
	 * Method to get a control group with label and input.
	 *
	 * @param   array  $options  Options to be passed into the rendering of the field
	 *
	 * @return  string  A string containing the html for the control group
	 *
	 * @since   3.7.3
	 */
	public function renderField($options = array())
	{
		$options['class'] = empty($options['class']) ? 'field-spacer' : $options['class'] . ' field-spacer';

		return parent::renderField($options);
	}
}