<?php
/**
* @package RSMediaGallery!
* @copyright (C) 2011 - 2021 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
*/

defined('_JEXEC') or die('Restricted access');

JFormHelper::loadFieldClass('list');

class JFormFieldTags extends JFormFieldList {
	
	protected $type = 'Tags';

	public function __construct($form = null) {
		parent::__construct($form);
		
		JHtml::_('jquery.framework');
		JHtml::_('formbehavior.chosen', 'select[id*="[rsmg-select]"]');
		
		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/helper.php';
		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/version.php';
		
		$document	= JFactory::getDocument();
		$version	= (string) new RSMediaGalleryVersion;
		
		JHtml::_('stylesheet', 'com_rsmediagallery/chosen-tags.css', array('relative' => true, 'version' => $version));
		
		// jQuery Chosen does not keep track of the selected values order
		$document->addScriptDeclaration("jQuery(document).ready(function() {
			jQuery('[id*=\"[rsmg-select]\"]').each(function() {
				var rsmg_tags_drop		= jQuery(this),
					rsmg_tags_chosen	= rsmg_tags_drop.parent().find('> .chzn-container, > .chosen-container'),
					rsmg_tags_input		= rsmg_tags_drop.parent().find('> input[type=\"hidden\"]'),
					rsmg_tags_val		= rsmg_tags_input.val().replace(/,\s*$/, ''),
					rsmg_tags_selected	= (rsmg_tags_val != '') ? rsmg_tags_val.split(',') : new Array();
				
				rsmg_tags_chosen.find('> ul.chzn-choices > li.search-choice, > ul.chosen-choices > li.search-choice').each(function(index) {
					jQuery(this).find('> span').text(rsmg_tags_selected[index]);
					jQuery(this).find('> .search-choice-close').attr('data-option-array-index', rsmg_tags_drop.find('> option[value=\"' + rsmg_tags_selected[index] + '\"]').index());
				});
				
				rsmg_tags_drop.on('change', function() {
					var rsmg_tags_drop_val = jQuery(this).val() ? jQuery(this).val() : new Array();
					
					if (rsmg_tags_drop_val.length > 0) {
						if (rsmg_tags_drop_val.length > rsmg_tags_selected.length) {
							jQuery.each(rsmg_tags_drop_val, function(key, value) {
								if (jQuery.inArray(value, rsmg_tags_selected) === -1) {
									rsmg_tags_selected.push(value);
								}
							});
						} else {
							if (rsmg_tags_selected.length > 1) {
								jQuery.each(rsmg_tags_selected, function(key, value) {
									if (jQuery.inArray(value, rsmg_tags_drop_val) === -1) {
										rsmg_tags_selected = jQuery.grep(rsmg_tags_selected, function(val) {
											return val != value;
										});
									}
								});
							} else {
								rsmg_tags_selected = [];
							}
						}
					} else {
						rsmg_tags_selected = [];
					}
					
					rsmg_tags_input.val(rsmg_tags_selected.join(','));
				});
			});
		});");
	}

	protected function getInput() {
		
		$html = '';
		$attr = '';
		
		// Initialize some field attributes.
		$attr .= !empty($this->class) ? ' class="' . $this->class . '"' : '';
		$attr .= !empty($this->size) ? ' size="' . $this->size . '"' : '';
		$attr .= ' multiple';
		
		// Initialize JavaScript field attributes.
		$attr .= $this->onchange ? ' onchange="' . $this->onchange . '"' : '';
		
		$selected		= array_filter(explode(',', trim($this->value)));
		$options		= (array) $this->getOptions();
		$non_existing	= array();
		
		// Remove unexisting previously saved tags
		if (count($selected)) {
			foreach ($selected as $sel_key => $sel_value) {
				if (!in_array($sel_value, $options)) {
					array_push($non_existing, $selected[$sel_key]);
					unset($selected[$sel_key]);
				}
			}
			$selected		= array_values($selected);
			$this->value	= implode(',', $selected);
		}
		
		if (count($options)) {
			$html .= '<select ' . 'id="' . $this->name . '[rsmg-select]"' . ' name="' . $this->name . '[rsmg-select]"' . $attr . '>';

			foreach ($options as $option) {
				$html .= '<option value="' . $option . '"' . (in_array($option, $selected) ? ' selected="selected"' : '') . '>' . $option . '</option>';
			}
			$html .= '</select>';
			
			if (count($non_existing)) {
				$html .= '<div class="alert alert-warning" style="margin-top: 15px;">' . JText::sprintf('COM_RSMEDIAGALLERY_PARAM_TAGS_NO_RESULTS', implode(', ', $non_existing)).'</div>';
			}
		} else {
			$html .= '<div class="alert alert-info">' . JText::_('COM_RSMEDIAGALLERY_PLEASE_ADD_TAG') . '</div>';
		}
		
		$html .= '<input type="hidden" name="' . $this->name . '" value="' . htmlspecialchars($this->value, ENT_COMPAT, 'UTF-8') . '"/>';
		
		return $html;
	}

	protected function getOptions() {
		
		$options = RSMediaGalleryHelper::getAllTags();

		// Merge any additional options in the XML definition.
		$options = array_merge(parent::getOptions(), $options);

		return $options;
	}
}