<?php
/**
* @package RSMediaGallery!
* @copyright (C) 2011 - 2021 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
*/

defined('_JEXEC') or die('Restricted access');

if (!class_exists('RSMediaGalleryHelper'))
	require_once dirname(__FILE__).'/helper.php';

class RSMediaGalleryIntegration
{
	protected $index 		= 0;
	protected $initialized	= false;
	protected $namespace 	= 'default';

	public static function getInstance() {
		static $inst;
		
		if (empty($inst))
			$inst = new RSMediaGalleryIntegration();
		
		return $inst;
	}

	protected function addScripts() {
		$document 	= JFactory::getDocument();
		$min		= JDEBUG ? '' : '.min';
		
		// Javascript
		$document->addScriptDeclaration('jQuery.noConflict();');
		$document->addScript(JUri::root(true).'/media/com_rsmediagallery/js/jquery.magnific-popup'.$min.'.js');
		$document->addScript(JUri::root(true).'/media/com_rsmediagallery/js/jquery.integration-script'.$min.'.js');
	}

	protected function addStyleSheets() {
		$document 		 = JFactory::getDocument();
		$document->addStyleSheet(JUri::root(true).'/media/com_rsmediagallery/css/magnific-popup.css');
	}

	protected function addInitScript() {
		$document = JFactory::getDocument();
		$document->addScriptDeclaration("jQuery(document).ready(function() {
			if (typeof rsmgIntegrationInitMagnificPopup == 'function') {
				rsmgIntegrationInitMagnificPopup('" . $this->namespace . "');
			}
		});");
	}

	protected function addOwnStyleSheet() {
		// this makes sense only when the namespace is 'default'
		// namespacing allows you to define your own layout when showing the gallery with your own stylesheet
		if ($this->namespace == 'default') {
			jimport('joomla.application.component.helper');
			
			$document			= JFactory::getDocument();
			$componentParams	= JComponentHelper::getParams('com_rsmediagallery');
			
			$document->addStyleSheet(JUri::root(true).'/media/com_rsmediagallery/css/integration-style.css');
			$document->addStyleSheet(JUri::root(true).'/media/com_rsmediagallery/css/integration-'.$componentParams->get('contrast', 'light').'.css');
		}
	}

	/* @param mixed $tags 		can be either array eg. array('tag1', 'tag2') or a string with comma separated values eg. 'tag1, tag2' *
	 * @param mixed $options 	array or object of options */
	public function display($tags, $options = array(), $namespace = 'default') {
		$html = '';
		$this->namespace = preg_replace('/(\s|[^A-Za-z0-9\-])+/', '_', $namespace);

		$params = new JObject();
		
		if (is_array($options)) {
			foreach ($options as $option => $value) {
				$params->set($option, $value);
			}
		} elseif (is_object($options)) {
			// already a JParameter or JObject or JRegistry?
			if (!is_callable(array($options, 'get'))) {
				foreach (get_object_vars($options) as $option => $value) {
					$params->set($option, $value);
				}
			} else {
				$params = $options;
			}
		}
		
		// convert thumb_width & thumb_height
		if (!$params->get('thumb_resolution')) {
			if ($params->get('thumb_width')) {
				$params->set('thumb_resolution', 'w,'.$params->get('thumb_width'));
			} elseif ($params->get('thumb_height')) {
				$params->set('thumb_resolution', 'h,'.$params->get('thumb_height'));
			} else {
				$params->set('thumb_resolution', 'w,300');
			}
		}
		// same for full_width & full_height
		if (!$params->get('full_resolution')) {
			if ($params->get('full_width')) {
				$params->set('full_resolution', 'w,'.$params->get('full_width'));
			} elseif ($params->get('full_height')) {
				$params->set('full_resolution', 'h,'.$params->get('full_height'));
			} else {
				$params->set('full_resolution', 'w,800');
			}
		}
		
		$params->set('thumb_width', 0);
		$params->set('thumb_height', 0);
		
		$params					= RSMediaGalleryHelper::parseParams($params);
		
		$order 					= $params->get('ordering', 'ordering');
		$direction 				= $params->get('direction', 'ASC');
		$limitstart 			= (int) $params->get('limitstart', 0);
		$limit					= (int) $params->get('limit', 0);
		$show_title				= (int) $params->get('show_title', 1);
		$show_description 		= (int) $params->get('show_description', 1);
		$show_details_thumb 	= (int) $params->get('show_details_thumb', 0);
		$itemsrow				= (int) $params->get('itemsrow', 3);
		$center_list			= (int) $params->get('center_list', 1);
		$use_original 			= (int) $params->get('use_original', 0);
		$image					= (int) $params->get('image', 0);
		$thumb_max_size			= $params->get('thumb_width') ? ' style="max-width: ' . $params->get('thumb_width') . 'px;"' : ' style="max-height: ' . $params->get('thumb_height') . 'px;"';
		$thumb_container_height	= $params->get('thumb_width') ? '' : ' style="padding-top: ' . $params->get('thumb_height') . 'px;"';

		if (!empty($tags) && $items = RSMediaGalleryHelper::getItems($tags, $order, $direction, 0, $limit)) {
			if (!$this->initialized) {
				$this->addScripts();
				$this->addStyleSheets();
				$this->initialized = true;
			}
			
			$this->addOwnStyleSheet();
			$this->addInitScript();
			
			$html .= '<div class="rsmg-'.$this->namespace.'-main-container' . ($center_list ? ' rsmg-center': '') . '">';
			$html .= '<ul class="rsmg-'.$this->namespace.'-gallery">';
			
			$itemsrow = ($itemsrow > 6 ? 6 : ($itemsrow == 5 ? 4 : $itemsrow));
			
			foreach ($items as $i => $item) {
				$item = RSMediaGalleryHelper::parseItem($item, $params);

                $item_data_attributes = 'data-mfp-src="' . addslashes($item->full) . '"';

                $item_data_attributes .= ' data-item-id="' . $item->id . '"';
                $item_data_attributes .= ' data-item-title="' . ($show_title && !empty($item->title) ? $this->escape($item->title) : '') . '"';
                $item_data_attributes .= ' data-item-description="' . ($show_description && !empty($item->full_description) ? $item->full_description : '') . '"';

				$html .= '<li class="rsmg-col-' . $itemsrow . '">';
				$html .= '<div class="rsmg-'.$this->namespace.'-item-container"'.$thumb_max_size.'>';
				$html .= '<a class="rsmg-'.$this->namespace.'-item-link" href="'.addslashes($item->full).'"'.$item_data_attributes.'>';
                $html .= '<div class="rsmg-'.$this->namespace.'-item-thumb-container"'.$thumb_container_height.'>';
                $html .= '<div class="rsmg-'.$this->namespace.'-item-thumb">';
                $html .= '<img src="'.$item->thumb.'" alt="'.$this->escape($item->title).'"/>';
                $html .= '</div>';
                $html .= '</div>';

                if ((($show_title && !empty($item->title)) || ($show_description && !empty($item->full_description))) && $show_details_thumb) {
                    $html .= '<div class="rsmg-'.$this->namespace.'-item-body">';

                    if ($show_title && !empty($item->title)) {
                        $html .= '<h3 class="rsmg-'.$this->namespace.'-item-title">';
                        $html .= $item->title;
                        $html .= '</h3>';
                    }
                    if ($show_description && !empty($item->full_description)) {
                        $html .= '<div class="rsmg-'.$this->namespace.'-item-description">';
                        $html .= $item->full_description;
                        $html .= '</div>';
                    }
                    $html .= '</div>';
                }

                $html .= '</a>';
				$html .= '</div>';
				$html .= '</li>';
			}
			$html .= '</ul>';
			$html .= '</div>';
			
			$this->index++;
		}
		
		return $html;
	}

	protected function escape($string) {
		return RSMediaGalleryHelper::escape($string);
	}

	public function getTagsList() {
		$db 	= JFactory::getDbo();
		$query 	= $db->getQuery(true);
		
		$query->select('DISTINCT('.$db->qn('tag').')')
			  ->from($db->qn('#__rsmediagallery_tags'))
			  ->order($db->qn('tag').' '.$db->escape('asc'));
		
		$db->setQuery($query);
		return $db->loadColumn();
	}
}