<?php
/**
* @package RSMediaGallery!
* @copyright (C) 2011 - 2021 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-2.0.html
*/

defined('_JEXEC') or die('Restricted access');

class com_rsmediagalleryInstallerScript
{
	public function preflight($type, $parent) {
		$app = JFactory::getApplication();
		
		$jversion = new JVersion();

		if (!$jversion->isCompatible('3.9.25')) {
			$app->enqueueMessage('Please upgrade to at least Joomla! 3.9.25 before continuing!', 'error');
			return false;
		}
		
		if (file_exists(JPATH_ADMINISTRATOR . '/components/com_rsmediagallery/helpers/version.php')) {
			require_once JPATH_ADMINISTRATOR . '/components/com_rsmediagallery/helpers/version.php';
		
			if (class_exists('RSMediaGalleryVersion'))
			{
				$version = (string) new RSMediaGalleryVersion;

				if (version_compare($version, '2.0.0', '<')) {
					$this->updated_version = true;
				}
			}
		}
		
		return true;
	}
	
	public function postflight($type, $parent) {
		$app = JFactory::getApplication();
		
		if ($type == 'uninstall') {
			return true;
		}
		$db			= JFactory::getDbo();
		$messages 	= array(
			'plg_installer' => false
		);
		
		// Get the source
		$source = $parent->getParent()->getPath('source');
		
		// Get a new installer
		$installer = new JInstaller();
		
		if ($installer->install($source.'/other/plg_installer')) {
			$query = $db->getQuery(true);
			$query->update('#__extensions')
				  ->set($db->qn('enabled').'='.$db->q(1))
				  ->where($db->qn('element').'='.$db->q('rsmediagallery'))
				  ->where($db->qn('type').'='.$db->q('plugin'))
				  ->where($db->qn('folder').'='.$db->q('installer'));
			$db->setQuery($query);
			$db->execute();
			
			$messages['plg_installer'] = true;
		}

		if ($type == 'update') {
			jimport('joomla.application.component.helper');
			
			$componentParams = JComponentHelper::getParams('com_rsmediagallery');
			
			$source = $parent->getParent()->getPath('source');
			$this->runSQL($source, 'install.sql');
			
			$db->setQuery("SHOW COLUMNS FROM #__rsmediagallery_items WHERE `Field`='free_aspect'");
			if (!$db->loadResult()) {
				$db->setQuery("ALTER TABLE `#__rsmediagallery_items` ADD `free_aspect` TINYINT(1) NOT NULL AFTER `params`");
				$db->execute();
			}

			$columns = $db->getTableColumns('#__rsmediagallery_thumbs', false);
			if ($columns['hash']->Key != 'MUL') {
				$db->setQuery("ALTER TABLE #__rsmediagallery_thumbs ADD INDEX (`hash`)");
				$db->execute();
			}

			$columns = $db->getTableColumns('#__rsmediagallery_effects', false);
			if ($columns['item_id']->Key != 'MUL') {
				$db->setQuery("ALTER TABLE #__rsmediagallery_effects ADD INDEX ( `item_id` ), ADD INDEX ( `item_id` , `hash` )");
				$db->execute();
			}
			
			// check for broken images
			$query = $db->getQuery(true);
			$query->select($db->qn('filename'))
			  ->from($db->qn('#__rsmediagallery_items'))
			  ->where($db->qn('params')." LIKE ".$db->q('%d:NAN%')." OR ".$db->qn('params')." LIKE ".$db->q('%d:INF%'));
			$db->setQuery($query);
			
			if ($items = $db->loadObjectList()) {
				foreach ($items as $item) {
					if (file_exists(JPATH_SITE.'/components/com_rsmediagallery/assets/gallery/original/'.$item->filename)) {
						unlink(JPATH_SITE.'/components/com_rsmediagallery/assets/gallery/original/'.$item->filename);
					}
				}
				
				// the delete query
				$db->setQuery("DELETE FROM #__rsmediagallery_items WHERE ".$db->qn('params')." LIKE ".$db->q('%d:NAN%')." OR ".$db->qn('params')." LIKE ".$db->q('%d:INF%'));
				$db->execute();
			}

			if (empty($componentParams->get('updated_version')) && !empty($this->updated_version)) {
				$componentParams->set('updated_version', '1');
				
				// save the updated_version parameter
				$componentId	= JComponentHelper::getComponent('com_rsmediagallery')->id;
				$table			= JTable::getInstance('extension');
				$table->load($componentId);
				$table->bind(array('params' => $componentParams->toString()));

				if (!$table->check()) {
					echo $table->getError();
					return false;
				}
				if (!$table->store()) {
					echo $table->getError();
					return false;
				}
				
				$app->enqueueMessage('<p>Starting with <strong>RSMediaGallery! v2.0</strong>, <strong>Slideshow Module</strong> is removed from the Downloads section of the component.</p><p>This module can be easily replaced with one of the other slideshow modules which will continue to be available: <strong>Advanced Slideshow Module</strong>, <strong>Responsive Slider Module</strong> or <strong>Responsive Slideshow Module</strong>.</p>', 'notice');
			}
		}

		// show message
		$this->showInstallMessage($messages);
	}
	
	public function uninstall($parent) {
		$plg_installer_id = 0;
		
		$db = JFactory::getDbo();
		$query = $db->getQuery(true);
		$query->select($db->qn('extension_id'))
			  ->from($db->qn('#__extensions'))
			  ->where($db->qn('element').'='.$db->q('rsmediagallery'))
			  ->where($db->qn('type').'='.$db->q('plugin'))
			  ->where($db->qn('folder').'='.$db->q('installer'));
		$db->setQuery($query);
		$plg_installer_id = (int) $db->loadResult();
		
		if (!empty($plg_installer_id)) {
			// Get a new installer
			$installer = new JInstaller();
			$installer->uninstall('plugin', $plg_installer_id, 1);
		}
	}
	
	protected function runSQL($source, $file) {
		$db			= JFactory::getDbo();
		$driver		= strtolower($db->name);
		
		$sqlfile	= $source.'/admin/sql/mysql/'.$file;
		
		if (file_exists($sqlfile)) {
			$buffer = file_get_contents($sqlfile);
			
			if ($buffer !== false) {
				$queries = $db->splitSql($buffer);
				
				foreach ($queries as $query) {
					$query = trim($query);
					
					if ($query != '' && $query[0] != '#') {
						$db->setQuery($query);
						
						if (!$db->execute()) {
							JError::raiseWarning(1, JText::sprintf('JLIB_INSTALLER_ERROR_SQL_ERROR', $db->stderr(true)));
						}
					}
				}
			}
		}
	}
	
	protected function explode($string) {
		$string = str_replace(array("\r\n", "\r"), "\n", $string);
		return explode("\n", $string);
	}
	
	protected function quoteImplode($array) {
		$db = JFactory::getDbo();
		foreach ($array as $k => $v) {
			$array[$k] = $db->quote($v);
		}
		
		return implode(',', $array);
	}
	
	protected function showInstallMessage($messages=array()) {
		$isJ4 = version_compare(JVERSION, '4.0', '>=');
		
		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/helper.php';
?>
<style type="text/css">
.version-history {
	margin: 0 0 2em 0;
	padding: 0;
	list-style-type: none;
}
.version-history > li {
	margin: 0 0 0.5em 0;
	padding: 0 0 0 4em;
}
.version-new,
.version-fixed,
.version-upgraded {
	float: left;
	font-size: 0.8em;
	margin-left: -4.9em;
	width: 4.5em;
	color: white;
	text-align: center;
	font-weight: bold;
	text-transform: uppercase;
	-webkit-border-radius: 4px;
	-moz-border-radius: 4px;
	border-radius: 4px;
}

.version-new {
	background: #7dc35b;
}
.version-fixed {
	background: #e9a130;
}
.version-upgraded {
	background: #61b3de;
}

.install-ok {
	background: #7dc35b;
	color: #fff;
	padding: 3px;
}

.install-not-ok {
	background: #E9452F;
	color: #fff;
	padding: 3px;
}

.rsmg-install-info {
	margin: 1rem 0 2rem 0;
}
.rsmg-install-info img {
	max-width: 100%;
}
</style>
<div class="<?php echo $isJ4 ? 'row' : 'row-fluid'; ?> rsmg-install-info">
	<div class="<?php echo $isJ4 ? 'col-md-3' : 'span3'; ?>">
		<img src="../media/com_rsmediagallery/images/rsmediagallery-box.png" alt="RSMediaGallery! Box" />
	</div>
	<div class="<?php echo $isJ4 ? 'col-md-9' : 'span9'; ?>">
		<p>Installer Plugin ...
			<?php if ($messages['plg_installer']) { ?>
			<b class="install-ok">Installed</b>
			<?php } else { ?>
			<b class="install-not-ok">Error installing!</b>
			<?php } ?>
		</p>
		
		<h2>Changelog v2.0.5</h2>
		<ul class="version-history">
			<li><span class="version-upgraded">Upg</span> "Crop Image in LightBox" option was added to Gallery and Albums layouts.</li>
		</ul>
		<a class="btn btn-large btn-primary" href="index.php?option=com_rsmediagallery">Start using RSMediaGallery!</a>
		<a class="btn btn-secondary" href="https://www.rsjoomla.com/support/documentation/rsmediagallery.html" target="_blank">Read the RSMediaGallery! User Guide</a>
		<a class="btn btn-secondary" href="https://www.rsjoomla.com/support.html" target="_blank">Get Support!</a>
	</div>
</div>
		<?php
	}
}