<?php
/**
* @package RSMediaGallery!
* @copyright (C) 2011 - 2021 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
*/

defined('_JEXEC') or die('Restricted access');

class RSMediaGalleryViewAlbum extends JViewLegacy {

	public function display($tpl = null) {

		// Verify that the menu item exists
		if (is_null(JFactory::getApplication()->getMenu()->getActive())) {
			JFactory::getApplication()->redirect(JUri::root());
		}

		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/helper.php';
		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/version.php';

		$mainframe 			= JFactory::getApplication();
		$document  			= JFactory::getDocument();
        $version	        = (string) new RSMediaGalleryVersion();
		$componentParams	= JComponentHelper::getParams('com_rsmediagallery');
		$params    			= $this->get('AlbumParams');
		
		// The page class suffix
		$this->classFx      = trim($this->escape($params->get('pageclass_sfx','')));

        // Add Javascript variable for Joomla! version
        $document->addScriptDeclaration('var rsmg_jversion = '.RSMediaGalleryHelper::getJoomlaVersion());

		JHtml::_('jquery.framework');

		$this->thumb_size				= $params->get('thumb_width') ? $params->get('thumb_width') : $params->get('thumb_height');
		$this->thumb_max_size			= $params->get('thumb_width') ? ' style="max-width: ' . $params->get('thumb_width') . 'px;"' : '';
		$this->thumb_container_height	= $params->get('thumb_width') ? '' : ' style="padding-top: ' . $params->get('thumb_height') . 'px;"';
		$this->itemsrow     			= $params->get('itemsrow');

		$this->loadingbar				= $params->get('loadingbar_type', 'none');

		// Update 1.7.5
		$this->itemsrow     			= ($this->itemsrow > 6 ? 6 : ($this->itemsrow == 5 ? 4 : $this->itemsrow));

        $document->addScriptDeclaration('var rsmg_items_per_row = '.$this->itemsrow);
		
		// Social sharing
		$this->share_facebook 	 = (int) $params->get('share_facebook', 0);
		$this->share_twitter 	 = (int) $params->get('share_twitter', 0);
		$this->share_pinterest 	 = (int) $params->get('share_pinterest', 0);

		$this->social_on_thumbs  = (int) $params->get('social_on_thumbs', 1);
		$this->social_on_details = (int) $params->get('social_on_details', 1);
		$this->social_sharing 	 =  $this->share_facebook || $this->share_twitter || $this->share_pinterest;
		
		// Javascript variables needed for Load More button
        $js_variables = "var rsmg_social_sharing_on_thumbs 	= ".(($this->social_sharing && $this->social_on_thumbs) ? 'true' : 'false').",\n";
        $js_variables .= "rsmg_social_sharing_on_details    = ".(($this->social_sharing && $this->social_on_details) ? 'true' : 'false').",\n";
        $js_variables .= "rsmg_sharing_facebook 			= ".($this->share_facebook ? 'true' : 'false').",\n";
        $js_variables .= "rsmg_sharing_twitter 			    = ".($this->share_twitter ? 'true' : 'false').",\n";
        $js_variables .= "rsmg_sharing_pinterest 			= ".($this->share_pinterest ? 'true' : 'false').";\n";
		
		$document->addScriptDeclaration($js_variables);

		$min = JDEBUG ? '' : '.min';
		
		$document->addScriptDeclaration('jQuery.noConflict();');
		$document->addScriptDeclaration("function rsmgGetRoot() { return '".addslashes(rtrim(JUri::root(), '/'))."';}");

        // Add language constants
        JText::script('COM_RSMEDIAGALLERY_LOAD_ALL');
        JText::script('COM_RSMEDIAGALLERY_LOAD_MORE');
        JText::script('COM_RSMEDIAGALLERY_DOWNLOAD');
        JText::script('COM_RSMEDIAGALLERY_SHARE_ON');
        JText::script('COM_RSMEDIAGALLERY_SHARE_ON_PINTEREST');
        JText::script('COM_RSMEDIAGALLERY_SHARE_ON_TWITTER');
        JText::script('COM_RSMEDIAGALLERY_SHARE_ON_FACEBOOK');

		$layout = $this->getLayout();

        // CSS
        JHtml::_('stylesheet', 'com_rsmediagallery/icons.css', array('relative' => true, 'version' => $version));
        JHtml::_('stylesheet', 'com_rsmediagallery/style.css', array('relative' => true, 'version' => $version));

        if ($this->loadingbar != 'none') {
            JHtml::_('stylesheet', 'com_rsmediagallery/loadingbar.css', array('relative' => true, 'version' => $version));
        }

        JHtml::_('stylesheet', 'com_rsmediagallery/'.$componentParams->get('contrast', 'light').'.css', array('relative' => true, 'version' => $version));

        // Javascript
        JHtml::_('script', 'com_rsmediagallery/jquery.filterizr.min.js', array('relative' => true, 'version' => $version));

        JHtml::_('script', 'com_rsmediagallery/jquery.script'.$min.'.js', array('relative' => true, 'version' => $version));

		if ($this->loadingbar != 'none') {
            JHtml::_('script', 'com_rsmediagallery/jquery.loadingbar'.$min.'.js', array('relative' => true, 'version' => $version));
		}
		
		$this->tag = $this->get('Tags');
		
		if ($layout == 'image') {
			$this->item         = $this->get('item');
			$this->currentUrl   = JUri::getInstance()->toString();

			// Not an item ?
			if (!$this->item) {
				JFactory::getApplication()->enqueueMessage(JText::_('COM_RSMEDIAGALLERY_NOT_FOUND'), 'error');

				return;
			}

			if ($params->get('use_original', 0)) {
                $this->item->src    = JUri::root() . '/components/com_rsmediagallery/assets/gallery/original/' . $this->item->filename;
            } else {
                $full_height        = $params->get('full_height', 600);
                $full_width         = $params->get('full_width', 800);
				$this->item->src    = RSMediaGalleryHelper::getImage($this->item, $full_width, $full_height, true);
			}

			if ($this->social_sharing) {
				if ($this->share_facebook) {
					$document->addCustomTag('<meta property="og:title" content="'.$this->escape($this->item->title).'"/>');
					$document->addCustomTag('<meta property="og:url" content="'.$this->escape(JUri::current()).'"/>');
					$document->addCustomTag('<meta property="og:image" content="'.$this->escape($this->item->src).'"/>');
					$document->addCustomTag('<meta property="og:description" content="'.$this->escape($this->item->description).'"/>');
				}
			}
			
			// Set page title
			$document->setTitle($document->getTitle().' - '.$this->item->title);
			
			// Set description
			if (!empty($this->item->description)) {
				$document->setDescription($this->item->description);
			}
			
			// Set breadcrumbs
			$pathway = $mainframe->getPathway();
			$pathway->addItem($this->tag, RSMediaGalleryRoute::_('index.php?option=com_rsmediagallery&view=album&tag='.urlencode($this->tag)));
			
			$pathway = $mainframe->getPathway();
			$pathway->addItem($this->item->title, '');
			
			// Assign variables to the layout
			$this->adjacent = $this->get('adjacentitems');
			$this->params   = $params;
			
			// Don't forget to increase the views
			$model = $this->getModel('album');
			$model->hitItem($this->item->id);
		} else {

            JHtml::_('stylesheet', 'com_rsmediagallery/magnific-popup.css', array('relative' => true, 'version' => $version));
            JHtml::_('script', 'com_rsmediagallery/jquery.magnific-popup'.$min.'.js', array('relative' => true, 'version' => $version));

			$open_in = $params->get('open_in', 'slideshow');

            if ($open_in == 'slideshow') {
                $document->addScriptDeclaration("
                function rsmgInitMagnificPopup() {
                
                    var item_id         = '',
                        items           = [],
                        resize          = ".(int) $params->get('resize_image_lightbox', 1).";
                        
                    jQuery('.rsmg-gallery .rsmg-item-link').magnificPopup({
                        type        : 'inline',
                        preloader   : false,
                        gallery     : {
                            enabled : true,
                            preload : 0
                        },
                        callbacks   : {
                            elementParse : function(item) {
                                var item_details_wrapper    = jQuery('<div>',   {'class' : 'rsmg-item-details'}),
                                    item_details_container  = jQuery('<div>',   {'class' : 'rsmg-details-container rsmg-loading'}),
                                    
                                    // Item image
                                    item_image_container    = jQuery('<div>',   {'class' : 'rsmg-details-image'}),
                                    item_image              = jQuery('<img>',   {'src'   : item.src}),
                                    
                                    // Item data
                                    item_body_container     = jQuery('<div>',   {'class' : 'rsmg-details-body'}),
                                    item_title              = jQuery('<h2>',    {
                                        'class' : 'rsmg-details-title',
                                        'text'  : item.el.attr('data-item-title')
                                    }),
                                    item_description        = jQuery('<div>',   {
                                        'class' : 'rsmg-details-description',
                                        'text'  : item.el.attr('data-item-description')
                                    }),
                                    item_body_top           = jQuery('<div>',   {'class'  : 'rsmg-details-top'}),
                                    item_body_bottom        = jQuery('<div>',   {'class'  : 'rsmg-details-bottom'}),
                                    
                                    // Item share
                                    item_share_facebook     = jQuery('<a>',     {
                                        'class'     : 'rsmg-share',
                                        'target'    : '_blank',
                                        'href'      : item.el.attr('data-share-facebook'),
                                        'title'     : Joomla.JText._('COM_RSMEDIAGALLERY_SHARE_ON_FACEBOOK')
                                    }),
                                    item_share_twitter      = jQuery('<a>',     {
                                        'class'     : 'rsmg-share',
                                        'target'    : '_blank',
										'href'		: item.el.attr('data-share-twitter'),
                                        'title'     : Joomla.JText._('COM_RSMEDIAGALLERY_SHARE_ON_TWITTER')
                                    }),
                                    item_share_pinterest    = jQuery('<a>',     {
                                        'class'         : 'rsmg-share',
                                        'target'        : '_blank',
										'href'			: item.el.attr('data-share-pinterest'),
                                        'title'         : Joomla.JText._('COM_RSMEDIAGALLERY_SHARE_ON_PINTEREST'),
                                        'data-pin-do'   : 'buttonPin'
                                    }),
                                    
                                    // Item info
                                    item_tags               = jQuery('<div>',   {
                                        'class' : 'rsmg-details-tags',
                                        'text'  : item.el.attr('data-item-tags')
                                    }),
                                    item_info_container     = jQuery('<div>',   {'class'  : 'rsmg-details-info'}),
                                    item_download           = jQuery('<a>',     {
                                        'class' : 'rsmg-details-download',
                                        'href'  : item.el.attr('data-item-download'),
                                        'title' : Joomla.JText._('COM_RSMEDIAGALLERY_DOWNLOAD')
                                    }),
                                    item_hits               = jQuery('<span>',  {
                                        'class' : 'rsmg-details-hits',
                                        'text'  : item.el.attr('data-item-hits')
                                    }),
                                    item_created            = jQuery('<span>',  {
                                        'class' : 'rsmg-details-created',
                                        'text'  : item.el.attr('data-item-created')
                                    }),
                                    item_modified           = jQuery('<span>',  {
                                        'class' : 'rsmg-details-modified',
                                        'text'  : item.el.attr('data-item-modified')
                                    }),
                                    
                                    // Icons
                                    icon_facebook           = jQuery('<i>',     {'class' : 'rsmg-social-icon-facebook'}),
                                    icon_twitter            = jQuery('<i>',     {'class' : 'rsmg-social-icon-twitter'}),
                                    icon_pinterest          = jQuery('<i>',     {'class' : 'rsmg-social-icon-pinterest'}),
                                    icon_tags               = jQuery('<i>',     {'class' : 'rsmg-icon-tags'}),
                                    icon_download           = jQuery('<i>',     {'class' : 'rsmg-icon-download'}),
                                    icon_hits               = jQuery('<i>',     {'class' : 'rsmg-icon-eye'}),
                                    icon_created            = jQuery('<i>',     {'class' : 'rsmg-icon-calendar'}),
                                    icon_modified           = jQuery('<i>',     {'class' : 'rsmg-icon-edit'}),
                                    
                                    // Loader
                                    item_details_loader     = jQuery('<span>',  {'class' : 'rsmg-details-loader'});
                                
                                // Item image
                                item_image_container.append(item_image);
                                
                                // Item tags
                                if (item_tags.html() != '') {
                                    item_tags.prepend(icon_tags);
                                    item_body_bottom.append(item_tags);
                                }
                                
                                // Item hits
                                if (item_hits.html() != '') {
                                    item_hits.prepend(icon_hits);
                                    item_info_container.append(item_hits);
                                }
                                
                                // Item created date
                                if (item_created.html() != '') {
                                    item_created.prepend(icon_created);
                                    item_info_container.append(item_created);
                                }
                                
                                // Item modified date
                                if (item_modified.html() != '') {
                                    item_modified.prepend(icon_modified);
                                    item_info_container.append(item_modified);
                                }
                                
                                if (item_info_container.html() != '') {
                                    item_body_bottom.append(item_info_container);
                                }
                                
                                // Item download
                                if (item_download.attr('href') != '') {
                                    item_download.prepend(icon_download);
                                    item_body_top.append(item_download);
                                }
                                
                                // Item socials
                                if (item_share_facebook.attr('href') != '') {
                                    item_share_facebook.append(icon_facebook);
                                    item_body_top.append(item_share_facebook);
                                }
                                if (item_share_twitter.attr('href') != '') {
                                    item_share_twitter.append(icon_twitter);
                                    item_body_top.append(item_share_twitter);
                                }
                                if (item_share_pinterest.attr('href') != '') {
                                    item_share_pinterest.attr('data-pin-do', 'buttonPin');
                                    item_share_pinterest.append(icon_pinterest);
                                    item_body_top.append(item_share_pinterest);
                                }
                                
                                // Item details
                                if (item_title.html() != '') {
                                    item_body_container.append(item_title);
                                }
                                if (item_description.html() != '') {
                                    item_body_container.append(item_description);
                                }
                                if (item_body_bottom.html() != '') {
                                    item_body_container.append(item_body_bottom);
                                }
                                
                                // Item structure
                                if (item_body_top.html() != '') {
                                    item_details_container.append(item_body_top);
                                }
                                if (item_image_container.html() != '') {
                                    item_details_container.append(item_image_container);
                                }
                                if (item_body_container.html() != '') {
                                    item_details_container.append(item_body_container);
                                }
                                
                                if (resize) {
                                    item_image.css('max-height', 'calc(100vh - 275px)');
                                } else {
                                    item_image.css('max-height', item.el.attr('data-max-height') + 'px');
                                }
                                item_image.css('max-width', item.el.attr('data-max-width') + 'px');" . ($params->get('crop_image_lightbox', 1) ? '' : "item_image.css('width', '100%');") . "
                                item_details_wrapper.css('max-width', item.el.attr('data-max-width') + 'px');
                                
                                item_details_wrapper.append(item_details_container);
                                item_details_wrapper.append(item_details_loader);
                                
                                item.src = item_details_wrapper;
                            },
                            open        : function() {
                                var current_item = jQuery.magnificPopup.instance.st.el;
                                
                                rsmgHitItem(current_item.attr('data-item-id'));
                            },
                            change      : function() {                            
                                jQuery(this.content).find('img').on('load', function() {
                                    jQuery(this).closest('.rsmg-details-container').removeClass('rsmg-loading');
                                    jQuery(this).closest('.rsmg-details-container').find('~ .rsmg-details-loader').remove();
                                });
                            }
                        }
                    });
                }
                
				");
            }

			$items = $this->get('items');

			if ($items) {
				foreach ($items as $i => $item) {
                    $items[$i] = RSMediaGalleryHelper::parseItem($item, $params, true);
                }

				RSMediaGalleryHelper::addTags($items);
			}
			
			// Set breadcrumbs
			$pathway = $mainframe->getPathway();
			$pathway->addItem($this->tag, '');
			
			// Assign variables to the layout
			$this->params 		= $params;
			$this->items 		= $items;
			$this->total 		= $this->get('total');
			$this->limitstart 	= $this->get('limitstart');
			$this->limit 		= $this->get('limit');
			$this->more 		= ($this->limit > 0 ? ($this->limitstart + $this->limit < $this->total) : false);
			$this->prev			= $this->limitstart > 0;
			$this->itemid		= $this->get('itemid');
			
			$this->_prepareDocument();
		}
		
		parent::display($tpl);
	}
	
	protected function buildTwitterLink($link) {
		$link = urlencode($link);
		$link = str_replace('amp%3B', '', $link);
		
		return $link;
	}
	
	protected function _prepareDocument() {
		$title = $this->tag;
		$this->document->setTitle($title);
	}
}