<?php
/**
* @package RSMediaGallery!
* @copyright (C) 2011 - 2021 www.rsjoomla.com
* @license GPL, http://www.gnu.org/licenses/gpl-3.0.html
*/

defined('_JEXEC') or die('Restricted access');

class RSMediaGalleryViewAlbums extends JViewLegacy {

	public function display($tpl = null) {

		// verify that the menu item exists
		if (is_null(JFactory::getApplication()->getMenu()->getActive())) {
			JFactory::getApplication()->redirect(JUri::root());
		}
		
		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/helper.php';
		require_once JPATH_ADMINISTRATOR.'/components/com_rsmediagallery/helpers/version.php';

		$mainframe 			= JFactory::getApplication();
		$document  			= JFactory::getDocument();
        $version	        = (string) new RSMediaGalleryVersion;
		$componentParams	= JComponentHelper::getParams('com_rsmediagallery');
		$params    			= $this->get('params');

		$this->itemsrow     = $params->get('itemsrow');
		
		// the page class suffix
		$this->classFx      = trim($this->escape($params->get('pageclass_sfx','')));
		
		// update 1.7.5
		$this->itemsrow     = ($this->itemsrow > 6 ? 6 : ($this->itemsrow == 5 ? 4 : $this->itemsrow));

        $min                = JDEBUG ? '' : '.min';

        // CSS
        JHtml::_('stylesheet', 'com_rsmediagallery/icons.css', array('relative' => true, 'version' => $version));
        JHtml::_('stylesheet', 'com_rsmediagallery/style.css', array('relative' => true, 'version' => $version));
        JHtml::_('stylesheet', 'com_rsmediagallery/'.($componentParams->get('contrast', 'light')).'.css', array('relative' => true, 'version' => $version));

        $allMenu            = $mainframe->getMenu();
		$activeMenu         = $allMenu->getActive();
		$this->albumName    = $activeMenu->title;
		
		$this->tags         = !empty($this->get('Tags')) ? $this->get('Tags') : RSMediaGalleryHelper::getAllTags();
		$model              = $this->getModel('albums');
		
		$items              = !empty($this->tags) ? $this->get('TagItems') : array();

		$this->preview_res  = $params->get('display_preview_resolution');
        $preview_res        = $this->preview_res[1];
		$album_image_w      = array();

		if (count($items)) {
			foreach ($items as $tag => $items_tag) {
				$nr_items_tag = count($items_tag);

				foreach ($items_tag as $i => $item) {

					if (($nr_items_tag == 3 && ($i == 0 || $i == 1)) || $nr_items_tag == 4) {
						$new_preview_res = $preview_res / 2;
					} else {
						$new_preview_res = $preview_res;
					}

					if (!is_array($item->params)) {
						$item_params = @unserialize($item->params);
					}

					$original_width         = $item_params['selection']['x2'] - $item_params['selection']['x1'];
					$original_height        = $item_params['selection']['y2'] - $item_params['selection']['y1'];
					$reference_size         = $original_width > $original_height ? $original_height : $original_width;
					$ratio                  = $new_preview_res / $reference_size;
					$inner_thumb_width      = round($original_width * $ratio);
					$inner_thumb_height     = round($original_height * $ratio);

					$items[$tag][$i]        = $model->parseItemAlbum($item, $inner_thumb_width, $inner_thumb_height);
					$album_image_w[$tag]    = $inner_thumb_width;
				}
			}
		}
		
		if (count($album_image_w) > 0) {
			$album_image_w = json_encode($album_image_w);
			
			$document->addScriptDeclaration('var album_image_data = {};
			album_image_data = '.$album_image_w.';');
		}
		
		$this->items    = $items;
		
		$this->links    = $model->buildAlbumLinks($this->tags);
		$this->params   = $params;
		
		$this->_prepareDocument();	
		
		parent::display($tpl);
	}
	
	protected function _prepareDocument() {
		$app	= JFactory::getApplication();
		$menus	= $app->getMenu();
		$title	= null;

		// Because the application sets a default page title,
		// we need to get it from the menu item itself
		$menu   = $menus->getActive();

		if ($menu) {
			$this->params->def('page_heading', $this->params->get('page_title', $menu->title));
		} else {
			$this->params->def('page_heading', JText::_('COM_RSMG_ALBUMS'));
		}
		
		$title = $this->params->get('page_title', '');

		if (empty($title)) {
			$title = $app->get('sitename');
		} else if ($app->get('sitename_pagetitles', 0) == 1) {
			$title = JText::sprintf('JPAGETITLE', $app->get('sitename'), $title);
		} else if ($app->get('sitename_pagetitles', 0) == 2) {
			$title = JText::sprintf('JPAGETITLE', $title, $app->get('sitename'));
		}

		$this->document->setTitle($title);

		if ($this->params->get('menu-meta_description')) {
			$this->document->setDescription($this->params->get('menu-meta_description'));
		}

		if ($this->params->get('menu-meta_keywords')) {
			$this->document->setMetadata('keywords', $this->params->get('menu-meta_keywords'));
		}

		if ($this->params->get('robots')) {
			$this->document->setMetadata('robots', $this->params->get('robots'));
		}
	}
}